import sys, os
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'src')))

from playoff import Playoff, PlayoffException

client_id = 'clientid'
client_secret  ='clientsecret'

def test_wrong_init():
  try:
    pl = Playoff(
      version = 'v1',
      client_id = "wrong_id",
      client_secret = "wrong_secret",
      type = 'client'
    )
  except PlayoffException as e:
    assert 'Client authentication failed' in e.message

def test_v1():
  pl = Playoff(
    version = 'v1',
    client_id = client_id,
    client_secret = client_secret,
    type = 'client'
  )

  try:
    pl.get(route= '/gege', query= { 'player_id': 'student1' })
  except PlayoffException as e:
    assert 'route does not exist' in e.message

  players = pl.api(method = 'GET', route = '/players', query = { 'player_id': 'student1', 'limit': 1 })
  assert players['data'] != None
  assert players['data'][0] != None

  try:
    pl.get(route = '/player')
  except PlayoffException as e:
    assert "The 'player_id' parameter should be specified in the query" in e.message

  player_id = 'student1'
  player = pl.get(route = '/player', query = { 'player_id': player_id } )
  assert player["id"] == "student1"
  assert player["alias"] == "Student1"
  assert player["enabled"] == True

  pl.get(route = '/definitions/processes', query = { 'player_id': player_id } )
  pl.get(route ='/definitions/teams', query = { 'player_id': player_id } )
  pl.get(route = '/processes', query = { 'player_id': player_id } )
  pl.get(route = '/teams', query = { 'player_id': player_id } )

  processes = pl.get(route = '/processes', query = { 'player_id': 'student1', 'limit': 1, 'skip': 4 })
  assert processes["data"][0]["definition"] == "module1"
  assert len(processes["data"]), 1

  new_process = pl.post(route = '/definitions/processes/module1', query = { 'player_id': player_id })
  assert new_process["definition"] == "module1"
  assert new_process["state"] == "ACTIVE"

  patched_process = pl.patch(
    route = "/processes/%s" %new_process['id'],
    query = { 'player_id': player_id },
    body = { 'name': 'patched_process', 'access': 'PUBLIC' }
  )

  assert patched_process['name'] == 'patched_process'
  assert patched_process['access'] == 'PUBLIC'

  deleted_process = pl.delete(route = "/processes/%s" %new_process['id'], query = { 'player_id': player_id })
  assert deleted_process['message'] != None

  raw_data = pl.get(route = '/player', query = { 'player_id': player_id }, raw = True)
  assert type(raw_data) is str

def test_v2():
  pl = Playoff(
    version = 'v2',
    client_id=client_id,
    client_secret=client_secret,
    type = 'code'
  )

  try:
    pl.get(route= '/gege', query= { 'player_id': 'student1' })
  except PlayoffException as e:
    assert 'route does not exist' in e.message

  players = pl.api(method = 'GET', route = '/runtime/players', query = { 'player_id': 'student1', 'limit': 1 })
  assert players['data'] != None
  assert players['data'][0] != None

  try:
    pl.get(route = '/runtime/player')
  except PlayoffException as e:
    assert "The 'player_id' parameter should be specified in the query" in e.message

  player_id = 'student1'
  player = pl.get(route = '/runtime/player', query = { 'player_id': player_id } )
  assert player["id"] == "student1"
  assert player["alias"] == "Student1"
  assert player["enabled"] == True

  pl.get(route = '/runtime/definitions/processes', query = { 'player_id': player_id } )
  pl.get(route ='/runtime/definitions/teams', query = { 'player_id': player_id } )
  pl.get(route = '/runtime/processes', query = { 'player_id': player_id } )
  pl.get(route = '/runtime/teams', query = { 'player_id': player_id } )

  processes = pl.get(route = '/runtime/processes', query = { 'player_id': 'student1', 'limit': 1, 'skip': 4 })
  assert processes["data"][0]["definition"] == "module1"
  assert len(processes["data"]), 1

  new_process = pl.post(route = '/runtime/processes', query = { 'player_id': player_id }, body = { 'definition': 'module1' })
  assert new_process["definition"]['id'] == "module1"
  assert new_process["state"] == "ACTIVE"

  patched_process = pl.patch(
    route = "/runtime/processes/%s" %new_process['id'],
    query = { 'player_id': player_id },
    body = { 'name': 'patched_process', 'access': 'PUBLIC' }
  )

  assert patched_process['name'] == 'patched_process'
  assert patched_process['access'] == 'PUBLIC'

  deleted_process = pl.delete(route = "/runtime/processes/%s" %new_process['id'], query = { 'player_id': player_id })
  assert deleted_process['message'] != None

  raw_data = pl.get(route = '/runtime/player', query = { 'player_id': player_id }, raw = True)
  assert type(raw_data) is str


def store(access_token):
  print('Storing')
  print(access_token)

def test_store():
  pl = Playoff(
    version = 'v1',
    client_id=client_id,
    client_secret=client_secret,
    type = 'client',
    store = store
  )

def test_jwt():
  token = Playoff.createJWT(
    client_id=client_id,
    client_secret=client_secret,
    player_id='fbatista@deloitte.pt'
  )
  print("token-> " )
  print(str(token))

def test_auth():
  try:
    pl = Playoff(
      version = 'v1',
      client_id=client_id,
      client_secret=client_secret,
      type = 'code'
    )
  except PlayoffException as e:
    assert e.name == 'init_failed'

  pl = Playoff(
    version = 'v1',
    client_id=client_id,
    client_secret=client_secret,
    type = 'code',
    redirect_uri = 'https://localhost:3000/auth/callback'
  )
  print(str(pl.get_login_url()))

def get_player(player_id):
    
  return player

def get_all_players():
  pl = Playoff(
    client_id=client_id,
    client_secret=client_secret,
    type='client'
    ,allow_unsecure=True
  )
  players = pl.get(
      route='/admin/players',
      query={}
  )
  return players

def get_action_template(action_id):
    pl = Playoff(
        client_id=client_id,
        client_secret=client_secret,
        type='client'
        ,allow_unsecure=True
        ,hostname='playoffgenerali.it'
    )
    action = pl.get(
        route='/design/versions/latest/actions/'+action_id,
        query={}
    )
    return action

def create_challenge(challenge_id):
    pl = Playoff(
        client_id=client_id,
        client_secret=client_secret,
        type='client'
    )
    challenge_template_id =  'challenge_template'
    action = get_action_template(challenge_template_id)
    action['id'] = challenge_id
    action['name'] = 'ch1_gen'
    action['rules'][0]['rewards'][0]['value'] = "110"
    action['rules'][0]['rewards'][1]['value'] = "120"
    action['rules'][0]['rewards'][2]['value'] = "130"
    action['rules'][1]['rewards'][0]['value'] = "0"
    action['rules'][1]['rewards'][1]['value'] = "0"
    action['rules'][1]['rewards'][2]['value'] = "0"

    action = pl.post(
        route='/design/versions/latest/actions',
        body =action
    )
    return action

def update_challenge(challenge_id):
    pl = Playoff(
        client_id=client_id,
        client_secret=client_secret,
        type='client'
    )
    challenge_template_id =  'challenge_template'
    action = get_action_template(challenge_id)
    action.pop('id', None)#there cannot be an id for PATCH
    action['name'] = 'ch1_mod'
    action['rules'][0]['rewards'][0]['value'] = "1100"
    action['rules'][0]['rewards'][1]['value'] = "1200"
    action['rules'][0]['rewards'][2]['value'] = "1300"
    action['rules'][1]['rewards'][0]['value'] = "0"
    action['rules'][1]['rewards'][1]['value'] = "0"
    action['rules'][1]['rewards'][2]['value'] = "0"

    action = pl.patch(
        route = '/design/versions/latest/actions/'+challenge_id,
        body = action
    )
    return action

def delete_challenge(challenge_id):
    pl = Playoff(
        client_id=client_id,
        client_secret=client_secret,
        type='client'
    )
    challenge_template_id =  'challenge_tmp_max_01'
    action = get_action_template(challenge_template_id)
    action['id'] = challenge_id


    action = pl.delete(
        route='/design/versions/latest/actions/'+challenge_id
    )
    return action


test_jwt()


#

#print(get_player('supa'))
#print(get_all_players())
print (get_action_template('challenge_tmp_max_01'))
#ch print("ok")
#print(delete_challenge('sfida1'))
#print(create_challenge('sfida1'))
#print(update_challenge('sfida1'))




import socket
import urllib.request, urllib.error, urllib.parse
import urllib.request, urllib.parse, urllib.error
import json
import time
import jwt
import datetime
from urllib.error import URLError, HTTPError

import ssl



socket.setdefaulttimeout(60)

class PlayoffException(Exception):

  def __init__(self, error, error_description):
    self.name = error
    self.message = error_description
    self.msg = error_description

  def __str__(self):
    return "%s %s" %(self.name, self.message)

class Playoff:
  hostname = 'playoffgamification.io'
  client_id = ''
  client_secret = ''
  type = ''
  store = None
  load = None
  redirect_uri = ''
  code = None

  @staticmethod
  def createJWT(client_id, client_secret, player_id, scopes = [], expires = 3600):
    token = jwt.encode({'player_id': player_id, 'scopes': scopes, 'exp': datetime.datetime.utcnow() + datetime.timedelta(seconds=expires)}, client_secret, algorithm='HS256')
    token = client_id + ':' + str(token.decode('utf8'))
    return token

  def __init__(self, client_id, client_secret, type, redirect_uri='', store=None, load=None, version='v2', hostname='playoffgamification.io', allow_unsecure=False):
    self.version = version
    self.client_id = client_id
    self.client_secret = client_secret
    self.type = type
    self.store = store
    self.load = load
    self.hostname = hostname
    if allow_unsecure:
      ssl._create_default_https_context = ssl._create_unverified_context
    if store == None:
      self.store = lambda access_token: ''

  def get_access_token(self):
    headers = { 'Accept': 'text/json'}
    if self.type == 'client':
      data = urllib.parse.urlencode({ 'client_id': self.client_id, 'client_secret': self.client_secret, 'grant_type': 'client_credentials'}).encode("utf-8")
    else:
      data = urllib.parse.urlencode({ 'client_id': self.client_id, 'client_secret': self.client_secret,
        'grant_type': 'authorization_code', 'redirect_uri': self.redirect_uri, 'code': self.code
      }).encode("utf-8")
    req = urllib.request.Request('https://'+self.hostname+'/auth/token', data, headers)
    try:
      response = urllib.request.urlopen(req).read()
    except URLError as e:
      err = json.loads(e.read())
      e.close()
      raise PlayoffException(err['error'], err['error_description'])
    token =json.loads(response)
    token['expires_at'] = int(round(time.time())) + int(token['expires_in'])
    del token['expires_in']
    self.store(token)
    if self.load == None:
      self.load = lambda: token

  def api(self, method='GET', route='', query = {}, body={}, raw=False, retry_flag=False):
    access_token = self.load()
    if int(round(time.time())) >= int(access_token['expires_at']):
      print('Access Token Expired')
      self.get_access_token()
      access_token = self.load()
    query['access_token'] = access_token['access_token']
    headers = { 'Accept': 'text/json', 'Content-Type': 'application/json' }
    req = urllib.request.Request("https://api."+ self.hostname +"/%s%s?%s" %(self.version, route, urllib.parse.urlencode(query)), json.dumps(body).encode("utf-8"), headers)
    req.get_method = lambda: method.upper()
    response = ''
    try:
      response = urllib.request.urlopen(req)
      if raw == True:
        raw_data = response.read()
        response.close()
        return raw_data
      else:
        json_data = json.loads(response.read())
        response.close()
        return json_data
    except HTTPError as e:
      err = json.loads(e.read())
      e.close()
      if err['error'] == 'invalid_access_token':
        self.get_access_token()
        if not retry_flag:
          return self.api(method, route, query, body, raw, True)
      raise PlayoffException(err['error'], err['error_description'])
    except URLError as e:
      err = json.loads(e.read())
      e.close()
      raise PlayoffException(err['error'], err['error_description'])

  def get(self, route='', query={}, raw=False):
    return self.api('GET', route, query, {}, raw)

  def post(self, route='', query={}, body={}):
    return self.api('POST', route, query, body)

  def put(self, route='', query={}, body={}):
    return self.api('PUT', route, query, body)

  def patch(self, route='', query={}, body={}):
    return self.api('PATCH', route, query, body)

  def delete(self, route='', query={}, body={}):
    return self.api('DELETE', route, query)

  def get_login_url(self):
    query = urllib.parse.urlencode({ 'response_type': 'code', 'redirect_uri': self.redirect_uri, 'client_id': self.client_id }).encode("utf-8")
    return "https://" + self.hostname + "/auth?%s" %query

  def exchange_code(self, code):
    self.code = code
    self.get_access_token()



from setuptools import setup, find_packages
version = '0.7.6'
setup(
  name = 'Playoff',
  version = version,
  packages= ['playoff'],
  description='This is the official OAuth 2.0 Python client SDK for the Playoff API',
  long_description='''
    It supports the client_credentials and authorization code OAuth 2.0 flows.
    For a complete API Reference checkout [Playoff Developers](https://dev.playoffgamification.io/docs/api) for more information.
  ''',
  url='https://github.com/officina/playoff-python-sdk',
  download_url=f'https://github.com/officina/playoff-python-sdk/archive/{version}.tar.gz',
  author='Officina',
  author_email='support@playoffgamification.io',
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Programming Language :: Python :: 3.6'
  ],
  keywords='REST, Playoff API, Playoff SDK, Gamification'
)
